/***************************************************************************************
 * @file     reb_main.c
 * @version  V1.0
 * @date     31. March 2016
 *
 * @note
 * VORAGO Technologies 
 *
 * @note
 * Copyright (c) 2013-2016 VORAGO Technologies. 
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY 
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT. 
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES 
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL 
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
#include <stdio.h>
#include "va108xx.h"
#include "reb_log.h"
#include "reb_board.h"
#include "reb_adt75.h"
#include "driver_common.h"
#include "reb_max11619.h"
#include "reb_timer.h"
#include "gpio_va108xx.h"

#define AN_simple_Rx_Tx 1  
// #define AN_Int_Rx_Tx 1  

#define UARTA_TX_BUFFER_LEN 256 
#define UARTB_RX_BUFFER_LEN  256



//volatile uint32_t rxCommandASCII_index = 0;
//         char rxCommandASCII[RX_COMMAND_ASCII_LEN];


				 char uartRxBuf[UARTB_RX_BUFFER_LEN];
volatile char *uartRxBufHead=uartRxBuf;
volatile char *uartRxBufTail=uartRxBuf;

				 char uartTxBuf[UARTA_TX_BUFFER_LEN];
volatile char *uartTxBufHead=uartTxBuf;
volatile char *uartTxBufTail=uartTxBuf;
         char TxBuffNotEmpty, RxBuffNotEmpty  ;  

volatile uint8_t bounce_cnt = 0;

/***************************************************************************************
 *  Init_UARTA_PortA
 *  Purpose:  Initialize UARTA on PORTA[8] and PORTA[9] pins 
 *   Input - Baud rate   
 *   Return - 1 = success 
 *   Other parameters for UART:  Parity = off, Stop bits = 1, Flowcontrol = off  
 ****************************************************************************************/
#define OSC_FREQUENCY 50000000
uint32_t Init_UARTA_PortA(uint32_t BaudRate)
{
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= ( 
																					 CLK_ENABLE_UARTA  | CLK_ENABLE_IRQSEL |
																					 CLK_ENABLE_IOMGR | CLK_ENABLE_UTILITY | CLK_ENABLE_PORTIO | CLK_ENABLE_SYSTEM ); 
	VOR_IOCONFIG->PORTA[9] =  ((0x2)<<IOCONFIG_PORTA_FUNSEL_Pos); // FUNSEL = 2 for UARTA_TX
	VOR_IOCONFIG->PORTA[8] = ((0x3) << IOCONFIG_PORTA_FLTTYPE_Pos | (0x2)<<IOCONFIG_PORTA_FUNSEL_Pos); 
	                              // filter rx input with 3 counts of system clock
	
	VOR_UARTA->CLKSCALE= ( (OSC_FREQUENCY / (BaudRate*16))<< 6)\
                            | ( ((OSC_FREQUENCY % (BaudRate*16))*64+(BaudRate*8))\
														     /(BaudRate*16) );
	VOR_UARTA->FIFO_CLR |= (UARTA_FIFO_CLR_TXFIFO_Msk | UARTA_FIFO_CLR_TXFIFO_Msk); 
	VOR_UARTA->CTRL |= (0x3<<UARTA_CTRL_WORDSIZE_Pos) ; //set word size to 8 bits, by default parity is off and #stopbit set to 1	
	
	#ifdef AN_Int_Rx_Tx	
	VOR_UARTA->TXFIFOIRQTRG=3;
	VOR_IRQSEL->UART[0] = 17   ;  // route UARTA IRQ to NVIC input 17 
	NVIC_SetPriority((IRQn_Type)17,1);
  NVIC_EnableIRQ((IRQn_Type)17);	    //  NVIC enable 
	#endif 
	return 1;  
}
/***************************************************************************************
 *  Init_UARTB_PortB
 *  Purpose:  Initialize UARTB on PORTB20 & 21 pins 
 *   Input - Baud rate   
 *   Return - 1 = success 
 *   Other parameters for UART:  Parity = off, Stop bits = 1, Flowcontrol = off 
 ****************************************************************************************/
uint32_t Init_UARTB_PortB(uint32_t BaudRate)
{
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= ( 
																					 CLK_ENABLE_UARTB  | CLK_ENABLE_IRQSEL |
																					 CLK_ENABLE_IOMGR | CLK_ENABLE_UTILITY | CLK_ENABLE_PORTIO | CLK_ENABLE_SYSTEM ); 
	VOR_IOCONFIG->PORTB[21] =  ( (0x1)<<IOCONFIG_PORTB_FUNSEL_Pos); // FUNSEL = 1 for UARTB_TX
	VOR_IOCONFIG->PORTB[20] = ((0x3) << IOCONFIG_PORTB_FLTTYPE_Pos | (0x1)<<IOCONFIG_PORTB_FUNSEL_Pos); 
	                                       // filter rx input with 3 counts of system clock
	
	VOR_UARTB->CLKSCALE= ( (OSC_FREQUENCY / (BaudRate*16))<< 6)\
                            | ( ((OSC_FREQUENCY % (BaudRate*16))*64+(BaudRate*8)) /(BaudRate*16) );
	VOR_UARTA->FIFO_CLR |= (UARTA_FIFO_CLR_TXFIFO_Msk | UARTA_FIFO_CLR_TXFIFO_Msk); 
	VOR_UARTB->CTRL |= (0x3<<UARTB_CTRL_WORDSIZE_Pos);// set word size to 8 bits, by default parity is off and #stopbit set to 1
	
	#ifdef AN_Int_Rx_Tx	
	VOR_UARTB->RXFIFOIRQTRG=10;
	VOR_UARTB->IRQ_ENB= UARTB_IRQ_ENB_IRQ_RX_Msk  ;  // enable interrupts 
	VOR_IRQSEL->UART[1] = 18   ;  // route UARTB IRQ to NVIC input 18 
	NVIC_SetPriority((IRQn_Type)18,1);
  NVIC_EnableIRQ((IRQn_Type)18);	    //  NVIC enable 
	#endif 
	return 1;  
}
/***************************************************************************************
 *  Tx_UARTA
 *  Purpose:  send data out  
 *   Input - number of bytes to send (must be < 16) and pointer to buffer head.    
 *   Return - 1 = success 
 ****************************************************************************************/
uint32_t Tx_UARTA(uint8_t Num_bytes, uint8_t *Tx_buff_ptr)  
{
uint8_t tx_count = 0   ; 

	while(tx_count < Num_bytes)      // load FIFO with data from buffer 
	{
	  VOR_UARTA->DATA = *Tx_buff_ptr++  ; 
	  tx_count++ ; 
	}
	
	VOR_UARTA->ENABLE = (1<<UARTA_ENABLE_TXENABLE_Pos)  ;  // enable UART TX 
	
	return 1;  
}

/***************************************************************************************
 *  ISR_Tx_UARTA
 *  Purpose:  send data out from Tx buffer until head pointer = tail pointer  
 *             load FIFO until it is full.  
 *   Input - called when half full mark reached     
 *   Return - none  (head_pointer is incremented once for each byte written to DATA register. 
 *    
 ****************************************************************************************/

void OC17_IRQHandler(void)
{
	  while((uartTxBufTail > uartTxBufHead) && (VOR_UARTA->TXSTATUS & UARTA_TXSTATUS_WRRDY_Msk)) // "1" indicates FIFO not full
		{ 
		   VOR_UARTA->DATA = *uartTxBufHead++  ;
		}
		if ( uartTxBufTail == uartTxBufHead) 
				{ 
					VOR_UARTA->IRQ_ENB = 0x00 ;  // end of data reached -> shut off ints 
				}
}

/***************************************************************************************
 *  ISR_Rx_UARTB
 *  Purpose:  move data from out from FIFO to Rx buffer until FIFO is empty  
 *   Input - none   
 *   Return - none  (head_pointer is incremented once for each byte written to DATA register. 
 *    
 ****************************************************************************************/

void OC18_IRQHandler (void)
{
			 if ((VOR_UARTB->RXSTATUS&UARTB_RXSTATUS_RDAVL_Msk) != 0 )
		   {
			     *uartRxBufTail++ = VOR_UARTB->DATA  ;
           if (uartRxBufTail - uartRxBuf > (UARTB_RX_BUFFER_LEN-1)) 
					 { uartRxBufTail = uartRxBuf;} // circular buffer - when reach end, point back to beginning	
						
				   RxBuffNotEmpty =  1 ;  //  set flag to inform main routine that data is available 

					 NVIC_ClearPendingIRQ((IRQn_Type)18);				
		   }
}
//***************************   Main routine is below this line  *************************
int main()
{
	uint8_t cur_temp = 0, UART_stat, UARTA_Tx_Buffer[16], UARTB_Rx_Buffer[32],*UARTA_Tx_Buffer_ptr, *UARTB_Rx_Buffer_ptr, temp ; 
	int32_t ret_code = 0;
	uint16_t  i;
volatile	uint32_t  count,tx_char_count  ;   //  counter for delay loop 
#ifdef GPIO_DRIVER
	VOR_DRIVER_GPIO *PA10 = &Driver_GPIOA_10; 
#endif 	
	
	/* Enable clock for all peripherals */ 
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE = ( CLK_ENABLE_PORTA | CLK_ENABLE_PORTB | CLK_ENABLE_SPIA | CLK_ENABLE_SPIB | CLK_ENABLE_SPIC |
																					 CLK_ENABLE_UARTA | CLK_ENABLE_UARTB | CLK_ENABLE_I2CA | CLK_ENABLE_I2CB | CLK_ENABLE_IRQSEL |
																					 CLK_ENABLE_IOMGR | CLK_ENABLE_UTILITY | CLK_ENABLE_PORTIO | CLK_ENABLE_SYSTEM ); 
	
				VOR_GPIO->BANK[0].DIR |= ( (1 << PORTA_10_D2) | (1 << PORTA_7_D3) );		//  Set outputs for LED2 & 3 on REB1 board 
	
#ifdef AN_simple_Rx_Tx	
	UART_stat = Init_UARTA_PortA(115000)  ;  
		if(UART_stat == 0)  { while(1)  ;  }  //  hang here if response is negative.  
	
	UART_stat = Init_UARTB_PortB(115000)  ;  
		if(UART_stat == 0)  { while(1)  ;  }  //  hang here if response is negative. 
		
		UARTA_Tx_Buffer[0] = 'V' ;  // Fill FIFO with 12 bytes
		UARTA_Tx_Buffer[1] = 'A'  ; 
		UARTA_Tx_Buffer[2] = '1'  ; 
		UARTA_Tx_Buffer[3] = '0'  ; 
		UARTA_Tx_Buffer[4] = '8'  ; 
		UARTA_Tx_Buffer[5] = 'x'  ; 
		UARTA_Tx_Buffer[6] = '0'  ; 
		UARTA_Tx_Buffer[7] = '_'  ; 
		UARTA_Tx_Buffer[8] = 'U'  ; 
		UARTA_Tx_Buffer[9] = 'A'  ; 
		UARTA_Tx_Buffer[10] = 'R'  ; 
		UARTA_Tx_Buffer[11] = 'T'  ; 
		
	  UARTA_Tx_Buffer_ptr = UARTA_Tx_Buffer  ;  
		
		UARTB_Rx_Buffer_ptr = UARTB_Rx_Buffer  ;
		VOR_UARTB->ENABLE = (1<<UARTB_ENABLE_RXENABLE_Pos)  ;  // enable receiver on UARTB  
		UART_stat = Tx_UARTA(12, UARTA_Tx_Buffer_ptr)  ;     // transmit 12 bytes of data out UARTA  
		
		while(1)
		{
		   if ((VOR_UARTB->RXSTATUS&UARTB_RXSTATUS_RDAVL_Msk) != 0 )
		   {
			     *UARTB_Rx_Buffer_ptr++ = VOR_UARTB->DATA  ;
           if (UARTB_Rx_Buffer_ptr - UARTB_Rx_Buffer > 32) 
								{UARTB_Rx_Buffer_ptr = UARTB_Rx_Buffer;} // always keep RX buffer size less than 32 		 
		   }	
	  }
		
#endif 	

#ifdef AN_Int_Rx_Tx
	UART_stat = Init_UARTA_PortA(115000)  ;  
		if(UART_stat == 0)  { while(1)  ;  }  //  hang here if response is negative  
	
	UART_stat = Init_UARTB_PortB(115000)  ;  
		if(UART_stat == 0)  { while(1)  ;  }  //  hang here if response is negative 
		
//		UARTA_Tx_Buffer[0] = 0x56  ; 
		for(i=0;i<93;i++)
		{
		   uartTxBuf[i] = 0x21+i ;   //  load 93 characters of sequential ASCII data into Tx buffer
    }			
		for(i=94;i<104;i++)
		{
		   uartTxBuf[i] = 0x24 ;   //  load 10 characters with ascii code for $
    }
		
//	  UARTA_Tx_Buffer_ptr = UARTA_Tx_Buffer  ;  
		
//		UARTB_Rx_Buffer_ptr = UARTB_Rx_Buffer  ;
		VOR_UARTB->ENABLE = (1<<UARTB_ENABLE_RXENABLE_Pos)  ;  // enable receiver on UARTB  
//		UART_stat = Tx_UARTA(16, UARTA_Tx_Buffer_ptr)  ;     // transmit 16 bytes of data out UARTA  
		
		 count = 0  ;  
		
			/* Intialize MSec and Sec Timers */
	ret_code = VOR_TIM0_MSecInitialize(10); 	//  set up interrupt to occur every 10 msec 
		
		
		tx_char_count = 1 ;  
		uartRxBufHead   =  uartRxBuf  ;    // initialize circular buffer head and tail pointer
		uartRxBufTail   =  uartRxBuf  ;  
		while(1)
		{
				count ++  ; 	

 		 if( VOR_TIM0_MSecExpired() )         //   This is 10 msec interval task 
     {
			VOR_GPIO->BANK[0].DATAOUT ^= (1 << PORTA_10_D2);		//  Toggle LED on REB1 board 
			
			if( !(VOR_GPIO->BANK[0].DATAIN & (1 << PORTA_11_SW_USER)) )  // Debounce user switch on REB1 board
				bounce_cnt++; 
			
			if(bounce_cnt >= 20) {     //  toggle LED3 if valid button press recognized
				VOR_GPIO->BANK[0].DATAOUT ^= (1 << PORTA_7_D3); 
				bounce_cnt = 0; 
			}
			
			while ((VOR_UARTB->RXSTATUS&UARTB_RXSTATUS_RDAVL_Msk) != 0 )  // check UARTB RX for data
				                      //   need to do this to check on case where #char < FIFO half full level
			{
					*uartRxBufTail++ = VOR_UARTB->DATA  ;
				  RxBuffNotEmpty =  1 ;  //  set flag to inform main routine that data is available
				  if (uartRxBufTail - uartRxBuf > (UARTB_RX_BUFFER_LEN-1)) 
					   { uartRxBufTail = uartRxBuf;} // circular buffer - when reach end, point back to beginning	
			}
			
			if (RxBuffNotEmpty != 0 )   //  process data in Rx Buffer.  (just read and discard) 
			{
				  while (uartRxBufTail != uartRxBufHead) 
					{
				   temp = *uartRxBufTail   ;  
					 uartRxBufHead ++ ; 
           if (uartRxBufHead - uartRxBuf > UARTB_RX_BUFFER_LEN-1) 
					    { uartRxBufHead = uartRxBuf;} // circular buffer - when reach end, point back to beginning						
					}
				  RxBuffNotEmpty = 0  ;  
			}
		
			VOR_TIM0_MSecClear(); 
		}  //  this is end of 10 msec task
			
 		    if(count > 100000/3)   //  occurs approximately every 14 msec  
		    {
					tx_char_count = tx_char_count + 1  ;  
					if(tx_char_count > 104) { tx_char_count = 5  ; }  
          uartTxBufHead=uartTxBuf;   // set head pointer to start 
          uartTxBufTail=uartTxBuf+tx_char_count;  // set tail to different lengths 
					VOR_UARTA->FIFO_CLR |= UARTA_FIFO_CLR_TXFIFO_Msk ; 
					VOR_UARTA->ENABLE = (1<<UARTA_ENABLE_TXENABLE_Pos)  ;  // enable UARTA TX 
          VOR_UARTA->IRQ_ENB= UARTA_IRQ_ENB_IRQ_TX_Msk  ;  // enable interrupts 					
					
					count = 0 ; 
		    }		
	  }   //  foot of while(1) 

#endif 
}	  //  foot of main loop 		
	
	
